{-# LANGUAGE ImportQualifiedPost #-}
{-# LANGUAGE TypeApplications #-}
{-# LANGUAGE ViewPatterns #-}

module Main (main) where

import Data.Text qualified as T
import Data.Text.Read (decimal)

parseVal :: [T.Text] -> (Integer, String)
parseVal [decimal @Integer -> Right cnt, name] = (fst cnt, T.unpack name)
parseVal _ = error "parseVal error"

parseVals :: T.Text -> [(Integer, String)]
parseVals vals = parseVal . T.split (== ' ') <$> T.splitOn (T.pack ", ") vals

parseParts :: [T.Text] -> (Integer, [[(Integer, String)]])
parseParts [T.words -> [T.unpack -> "Game", decimal @Integer -> Right d], values] = (fst d, parseVals <$> T.splitOn (T.pack "; ") values)
parseParts _ = error "parseParts error"

mapCubes :: T.Text -> (Integer, [[(Integer, String)]])
mapCubes line = parseParts $ T.splitOn (T.pack ": ") line

cubeAmtIsValid :: (Integer, String) -> Bool
cubeAmtIsValid (a, "red") = a <= 12
cubeAmtIsValid (a, "green") = a <= 13
cubeAmtIsValid (a, "blue") = a <= 14
cubeAmtIsValid _ = error "cubeAmtIsValid error"

main :: IO ()
main = do
  lns <- lines <$> getContents
  let values = mapCubes . T.pack <$> lns
  let filtered = filter (all (all cubeAmtIsValid) . snd) values

  print $ sum (fst <$> filtered)
