{-# LANGUAGE ViewPatterns #-}

module Main (main) where

import Data.Bifunctor (Bifunctor (bimap))
import Data.Char (isDigit)
import Data.List (singleton, transpose)
import Data.List.Extra (concatUnzip)

mergeChunks :: [(Char, a)] -> (String, [a])
mergeChunks = concatUnzip . fmap (bimap singleton singleton)

numberChunks :: [(Char, a)] -> [(String, [a])]
numberChunks [] = []
numberChunks ((isDigit -> False, _) : xs) = numberChunks xs
numberChunks items = (\(a, b) -> mergeChunks a : numberChunks b) $ span (isDigit . fst) items

main :: IO ()
main = do
  lns <- lines <$> getContents
  let len = length lns
  let offsetX x = drop (max x 0) . (++ replicate (max x 0) '.') . take len . (replicate (max (negate x) 0) '.' ++)
  let offsetY y = drop (max y 0) . (++ replicate (max y 0) (replicate len '.')) . take len . (replicate (max (negate y) 0) (replicate len '.') ++)
  let transposed = (transpose <$>) $ ((transpose <$>) <$>) $ transpose $ transpose <$> [[offsetY y $ offsetX x <$> lns | x <- [-1 .. 1]] | y <- [-1 .. 1]]
  let zipped = zipWith zip lns transposed
  let chunks = concatMap (((\(a, b) -> (read a :: Integer, b)) <$>) . numberChunks) zipped
  let filtered = filter (any (any (any (`notElem` ".01234567890"))) . snd) chunks
  print . sum $ fst <$> filtered
