package sh.natty.aoc23;

import org.apache.commons.lang3.mutable.MutableLong;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.*;
import java.util.function.Predicate;
import java.util.stream.Collectors;

record Mapping(long destination, long length) {
}

public class Main {
    public static void main(String[] args) {
        var reader = new BufferedReader(new InputStreamReader(System.in));
        var accum = new ArrayList<>(List.of(new ArrayList<String>()));
        var lines = reader.lines()
                .reduce(accum, (acc, val) -> {
                    if (val.isBlank())
                        acc.add(new ArrayList<>());
                    var last = acc.getLast();
                    last.add(val);
                    return acc;
                }, (l, r) -> {
                    l.addAll(r);
                    return l;
                });

        var seeds = Arrays.stream(lines.removeFirst()
                        .getFirst()
                        .split(":")[1]
                        .trim()
                        .split("\\s+"))
                .map(Long::parseLong)
                .toList();

        var maps = lines.stream().map(l -> new TreeMap<>(l.stream()
                .filter(Predicate.not(String::isBlank))
                .skip(1)
                .map(a -> Arrays.stream(a.split(" "))
                        .mapToLong(Long::parseLong)
                        .toArray())
                .collect(Collectors.toMap(k -> k[1], v -> new Mapping(v[0], v[2]))))
        ).toList();

        var locations = seeds.stream()
                .map(s ->
                        maps.stream()
                                .collect(() -> new MutableLong(s), (acc, v) -> {
                                    var e = v.floorEntry(acc.getValue());
                                    if (e == null)
                                        return;
                                    var val = e.getValue();
                                    var offs = acc.getValue() - e.getKey();
                                    if (offs > val.length())
                                        return;
                                    acc.setValue(val.destination() + offs);
                                }, MutableLong::setValue))
                .map(MutableLong::getValue)
                .toList();
        System.out.println(locations);

        System.out.println(locations.stream().min(Comparator.naturalOrder()));
    }
}
