package sh.natty.aoc23;

import org.apache.commons.lang3.NumberRange;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.*;
import java.util.function.Predicate;
import java.util.stream.Collectors;
import java.util.stream.IntStream;
import java.util.stream.Stream;

public class Main {
    record Mapping(long destination, long length) {
    }

    public static void main(String[] args) {
        var reader = new BufferedReader(new InputStreamReader(System.in));
        var accum = new ArrayList<>(List.of(new ArrayList<String>()));
        var lines = reader.lines()
                .reduce(accum, (acc, val) -> {
                    if (val.isBlank())
                        acc.add(new ArrayList<>());
                    var last = acc.getLast();
                    last.add(val);
                    return acc;
                }, (l, r) -> {
                    l.addAll(r);
                    return l;
                });

        var seeds = Arrays.stream(lines.removeFirst()
                        .getFirst()
                        .split(":")[1]
                        .trim()
                        .split("\\s+"))
                .map(Long::parseLong)
                .toList();

        var seedRanges = IntStream.range(0, seeds.size() / 2)
                .mapToObj(i -> new NumberRange<>(seeds.get(i * 2), seeds.get(i * 2) + seeds.get(i * 2 + 1) - 1, null))
                .toList();

        var maps = lines.stream().map(l -> new TreeMap<>(l.stream()
                .filter(Predicate.not(String::isBlank))
                .skip(1)
                .map(a -> Arrays.stream(a.split(" "))
                        .mapToLong(Long::parseLong)
                        .toArray())
                .collect(Collectors.toMap(k -> k[1], v -> new Mapping(v[0], v[2]))))
        ).toList();

        var ranges = seedRanges;
        for (var map : maps) {
            ranges = ranges.stream().flatMap(r -> {
                var minKey = map.floorKey(r.getMinimum());
                var maxKey = map.floorKey(r.getMaximum());

                if (minKey == null || maxKey == null)
                    return Stream.of(r);

                var sub = map.subMap(minKey, true, maxKey, true);

                var newRanges = new ArrayList<NumberRange<Long>>();
                var starts = new ArrayList<Long>();
                starts.add(r.getMinimum());
                var ends = new ArrayList<Long>();

                for (var e : sub.sequencedEntrySet()) {
                    var k = e.getKey();
                    var v = e.getValue();
                    var l = v.length();

                    if (k + l < r.getMinimum())
                        continue;

                    var ir = new NumberRange<>(k, k + l - 1, null);
                    var offset = v.destination() - k;

                    var intersection = ir.intersectionWith(r);
                    newRanges.add(new NumberRange<>(intersection.getMinimum() + offset, intersection.getMaximum() + offset, null));

                    ends.add(k - 1);
                    starts.add(k + l);
                }

                ends.add(r.getMaximum());

                var itStart = starts.iterator();
                var itEnd = ends.iterator();
                while (itStart.hasNext() && itEnd.hasNext()) {
                    var start = itStart.next();
                    var end = itEnd.next();

                    if (end < start)
                        continue;

                    var rr = new NumberRange<>(start, end, null);

                    var intersection = rr.intersectionWith(r);

                    newRanges.add(new NumberRange<>(intersection.getMinimum(), intersection.getMaximum(), null));
                }

                return newRanges.stream();
            }).toList();
        }

        System.out.println(ranges);
        System.out.println(ranges.stream().map(NumberRange::getMinimum).min(Comparator.naturalOrder()));
    }
}
