{-# LANGUAGE ViewPatterns #-}
{-# LANGUAGE ScopedTypeVariables #-}

module Main (main) where
import qualified Data.Text as T
import Data.List ( elemIndex, sort, sortBy, group, maximumBy )
import Data.Maybe (fromJust)
import GHC.Num (integerFromInt)
import Data.Function (on)
import Data.Ord (comparing, Down (Down))

cardStrength :: Char -> Integer
cardStrength = integerFromInt . fromJust . flip elemIndex (reverse "AKQT98765432J")

countTypes :: [Char] -> [Int]
countTypes = sortBy (comparing Down) . (length <$>) . group . sort

handStrength :: [Char] -> Integer
handStrength cc = case countTypes cc of
    [5] -> 6 -- 5 of a kind
    [4, 1] -> 5 -- 4 of a kind
    [3, 2] -> 4 -- full house
    [3, 1, 1] -> 3 -- 3 of a kind
    [2, 2, 1] -> 2 -- two pairs
    [2, 1, 1, 1] -> 1 -- a pair
    [1, 1, 1, 1, 1] -> 0 -- nothing
    _ -> error "Unreachable"

replaceJokers :: Char -> [Char] -> [Char]
replaceJokers c = fmap (\cc -> if cc == 'J' then c else cc)

maxJokerHand :: [Char] -> Char
maxJokerHand hand = maximumBy (compare `on` handStrength . flip replaceJokers hand) "AKQJT98765432"

maximizeHand :: [Char] -> [Char]
maximizeHand = replaceJokers =<< maxJokerHand

parseHand :: [T.Text] -> ([Char], Integer)
parseHand [T.unpack -> hand, read . T.unpack -> bid :: Integer] = (hand, bid)
parseHand _ = error "Parse error"

cmpHands :: [Char] -> [Char] -> Ordering
cmpHands = comparing (handStrength . maximizeHand)
    <> comparing (fmap cardStrength)

main :: IO ()
main = do
    hands <- (parseHand . T.words . T.pack <$>) . lines <$> getContents
    print $ sum . zipWith (*) [1..] $ snd <$> sortBy (cmpHands `on` fst) hands
