{-# LANGUAGE BinaryLiterals #-}
{-# LANGUAGE ImportQualifiedPost #-}

module Main (main) where

import Control.Applicative (Applicative (liftA2))
import Data.Bifunctor (first, second)
import Data.Bits (Bits (..))
import Data.Foldable (find, maximumBy)
import Data.Map qualified as M
import Data.Maybe (catMaybes, fromJust, fromMaybe)
import Data.Ord (comparing)
import GHC.Num (integerDiv)

parsePipe :: Char -> Integer
parsePipe = fromMaybe 0 . (M.fromList (zip "|-JLF7S" [0b1100, 0b0011, 0b1010, 0b1001, 0b0101, 0b0110, 0b1111]) M.!?)

flipDirs :: Integer -> Integer
flipDirs =
  sum
    . zipWith ($) (($ 1) . flip <$> [shiftL, shiftR, shiftL, shiftR])
    . zipWith (.&.) [0b0001, 0b0010, 0b0100, 0b1000]
    . repeat

main :: IO ()
main = do
  pipes <- M.fromList . concat . zipWith zip [[(x, y) | x <- [0 ..]] | y <- [0 ..]] . fmap (fmap parsePipe) . lines <$> getContents
  let findAdj c =
        fmap (\(aa, (bb, cc)) -> (bb, (0b1111 .&. (complement . flipDirs) aa) .&. cc))
          . filter (\(aa, (_, bb)) -> flipDirs aa .&. bb .&. (flipDirs . snd) c /= 0)
          . catMaybes
          . zipWith (liftA2 (,)) (pure <$> [0b0010, 0b0001, 0b1000, 0b0100])
          . fmap ((fmap . (,)) <*> (pipes M.!?))
          . (<$> ([first, second] <*> [(-1 +), (1 +)]))
          . flip id
          . fst
          $ c

  let start = fst . fromJust $ find ((== 0b1111) . snd) (M.assocs pipes)
  let path = takeWhile ((/= start) . fst) . tail $ iterate (head . filter ((/= 0) . snd) . findAdj) (start, 0b1111)
  print ((toInteger (length path) + 1) `integerDiv` 2)