{-# LANGUAGE BinaryLiterals #-}
{-# LANGUAGE ImportQualifiedPost #-}
{-# LANGUAGE ViewPatterns #-}

module Main (main) where

import Control.Applicative (Applicative (liftA2))
import Data.Bifunctor (first, second)
import Data.Bits (Bits (..))
import Data.Foldable (find)
import Data.Map qualified as M
import Data.Maybe (catMaybes, fromJust, fromMaybe)
import GHC.OldList (intercalate)

parsePipe :: Char -> Integer
parsePipe = fromMaybe 0 . (M.fromList (zip "|-JLF7S" [0b1100, 0b0011, 0b1010, 0b1001, 0b0101, 0b0110, 0b1111]) M.!?)

toPipe :: Integer -> Char
toPipe = fromMaybe '.' . (M.fromList (zip [0b1100, 0b0011, 0b1010, 0b1001, 0b0101, 0b0110, 0b1111] "┃━┛┗┏┓╋") M.!?)

flipDirs :: Integer -> Integer
flipDirs =
  sum
    . zipWith ($) (($ 1) . flip <$> [shiftL, shiftR, shiftL, shiftR])
    . zipWith (.&.) [0b0001, 0b0010, 0b0100, 0b1000]
    . repeat

main :: IO ()
main = do
  lns <- fmap (fmap parsePipe) . lines <$> getContents
  let pipes = M.fromList . concat . zipWith zip [[(x, y) | x <- [0 ..]] | y <- [0 ..]] $ lns
  let findAdj c =
        fmap (\(aa, (bb, cc)) -> (bb, (0b1111 .&. (complement . flipDirs) aa) .&. cc))
          . filter (\(aa, (_, bb)) -> flipDirs aa .&. bb .&. (flipDirs . snd) c /= 0)
          . catMaybes
          . zipWith (liftA2 (,)) (pure <$> [0b0010, 0b0001, 0b1000, 0b0100])
          . fmap ((fmap . (,)) <*> (pipes M.!?))
          . (<$> ([first, second] <*> [(-1 +), (1 +)]))
          . flip id
          . fst
          $ c

  let start = fst . fromJust $ find ((== 0b1111) . snd) (M.assocs pipes)
  let pipeStart = sum 
        $ flip (zipWith ((*) . toInteger . fromEnum)) [0b0010, 0b0001, 0b1000, 0b0100] 
        $ flip elem (fst <$> findAdj (start, 0b1111)) 
        <$> (($ start) <$> ([first, second] <*> [(-1 +), (1 +)])) 

  let path = 
        ((start, pipeStart) :)
        . takeWhile ((/= start) . fst) 
        . tail 
        $ iterate (head . filter ((/= 0) . snd) . findAdj) (start, pipeStart)

  let countSpaces = go 0 False
        where
            go _ v (0b1100 : xs) = go 0 (not v) xs
            go p v (0b0011 : xs) = go p v xs
            go _ v (0 : xs) = fromEnum v + go 0 v xs
            go p v (x : xs) 
              | p == 0 = go x v xs
              | p .&. x .&. 0b1100 > 0 = go 0 v xs
              | otherwise = go 0 (not v) xs
            go _ _ [] = 0

  let pathPipes = M.fromList path
  let piping = [[
          (\c -> if c == 0b1111 then pipeStart else c) 
          $ fromMaybe 0 
          $ pathPipes M.!? (x, y) 
          *> pipes M.!? (x, y) 
        | x <- [0 .. maximum (length <$> lns) - 1]] | y <- [0 .. length lns - 1]]

  putStrLn $ intercalate "\n" $ flip (zipWith (++)) (show . countSpaces <$> piping) $ fmap toPipe <$> piping
  print $ sum $ countSpaces <$> piping